@@ -21,8 +21,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#ifndef __EV_SIDEBAR_LINKS_H__
-#define __EV_SIDEBAR_LINKS_H__
+#pragma once
 
 #include <gtk/gtk.h>
 
@@ -60,7 +59,3 @@ GType      ev_sidebar_links_get_type       (void);
 GtkWidget *ev_sidebar_links_new            (void);
 
 G_END_DECLS
-
-#endif /* __EV_SIDEBAR_LINKS_H__ */
-
-
